//+---------------------------------------------------------------------------+
//| ALL RSI                                                                   |
//| Copyright  None, Public Domain                                           |
//|                                                                           |
//+---------------------------------------------------------------------------+
//| ORIGINAL AUTHOR: mladen                                                   |
//|        for Stochastics                                                    |
//|     Modified by: GodfreyH                                                 |
//|        for RSI                                                            |
//|     Modified by: Groenie                                                  |
//|        for B600                                                           |
//+---------------------------------------------------------------------------+
//| NOTES                                                                     |
//| 1. B600+ READY                                                            |
//+---------------------------------------------------------------------------+
//| TODO                                                                      |
//| 1. None                                                                   |
//+---------------------------------------------------------------------------+
//| CHANGELOG v1.1                                                            |
//| 1. Made indicator MT4 v6+ compatable                                      |
//| 2. Added overall & option descriptions                                    |
//| 3. Code cleanup & optimization                                            |
//+---------------------------------------------------------------------------+

//+---------------------------------------------------------------------------+
//|                              THE START                                    |
//+---------------------------------------------------------------------------+
#property copyright   "Public Domain"
#property description "Mini RSI charts for multi-timeframes"
#property link        "www.forex-tsd.com"
#property version     "1.1"
#define indicatorName "All_RSI"

//+---------------------------------------------------------------------------+
//| PROPERTIES                                                                |
//+---------------------------------------------------------------------------+
#property indicator_separate_window
#property indicator_minimum 0
#property indicator_maximum 100
#property indicator_buffers 1
#property indicator_color1  Gold
#property indicator_style1  STYLE_SOLID
#property indicator_level1  50
#property indicator_levelcolor Red

//---- CUSTOM PROPERTIES -----------------------------------------------------+
//-- External Properties --
extern string RP                   = "-------------------- | OPTIONS |";
extern int    RSIperiod            = 2;
extern string PF                   = "0=Price_Close 5=Typical_Price";
extern int    PriceField           = 0;  // 0=PRICE_CLOSE
extern string TF                   = "Chose timeframes (as in periodicity bar)";
extern string timeFrames           = "M1;M5;M15;M30;H1;H4;D1;W1;MN";
extern string BP                   = "Amount of bars to display for each window";
extern int    barsPerTimeFrame     = 25;
extern string SR                   = "Align mini charts to the right";
extern bool   shiftRight           = False;
extern string CF                   = "Show current timeframe before others";
extern bool   currentFirst         = False;
extern string TSC                  = "Text & Spearator Colors";
extern color  txtColor             = Silver; 
extern color  separatorColor       = DimGray; 
extern string EN                   = "Line colors can be chosen in Colors tab";
//+---------------------------------------------------------------------------+

//---- BUFFERS ---------------------------------------------------------------+
double ExtMapBuffer1[];
double ExtMapBuffer2[];
//+---------------------------------------------------------------------------+

//---- GLOBAL VARIABLES ------------------------------------------------------+
string shortName;
string labels[];
int    periods[];
int    Shift;
//+---------------------------------------------------------------------------+

//+---------------------------------------------------------------------------+
//| INIT()                                                                    |
//+---------------------------------------------------------------------------+
int init() {
      // Check if user wants bars shifted right
      if (shiftRight) {
         Shift = 1;
      } else {
         Shift = 0;
      }
      
      barsPerTimeFrame = MathMax(barsPerTimeFrame,15);      
      shortName = indicatorName+" ("+RSIperiod+")";
      IndicatorShortName(shortName);

      SetIndexBuffer(0,ExtMapBuffer1);
      SetIndexBuffer(1,ExtMapBuffer2);

      SetIndexShift(0,Shift*(barsPerTimeFrame+1));
      SetIndexShift(1,Shift*(barsPerTimeFrame+1));
      SetIndexLabel(0,"RSI");
      
      timeFrames = StringUpperCase(StringTrimLeft(StringTrimRight(timeFrames)));
      // Timeframe check
      if (StringSubstr(timeFrames,StringLen(timeFrames),1) != ";") {
         timeFrames = StringConcatenate(timeFrames,";");
      }                                  
            
      int s = 0;
      int i = StringFind(timeFrames,";",s);
      int time;
      string current;
      while (i > 0) {
         current = StringSubstr(timeFrames,s,i-s);
         time    = stringToTimeFrame(current);
         if (time > 0) {
               ArrayResize(labels ,ArraySize(labels)+1);
               ArrayResize(periods,ArraySize(periods)+1);
               labels[ArraySize(labels)-1] = current; 
               periods[ArraySize(periods)-1] = time;
         } // END if
         s = i + 1;
         i = StringFind(timeFrames,";",s);
      } // END while loop

      if(currentFirst)
         for (i=1;i<ArraySize(periods);i++)
         if (Period()==periods[i]) {
               string tmpLbl = labels[i];
               int    tmpPer = periods[i];
               
               for (int k=i ;k>0; k--) {
                  labels[k]  = labels[k-1];
                  periods[k] = periods[k-1];
               } // END for                   
               labels[0]  = tmpLbl;
               periods[0] = tmpPer;
         } // END if
         
//---- RETURN
return(0);

} // END INIT()

//+---------------------------------------------------------------------------+
//| DEINIT()                                                                  |
//+---------------------------------------------------------------------------+
int deinit() { 
   for(int l=0;l<ArraySize(periods);l++) {
         ObjectDelete(indicatorName+l);
         ObjectDelete(indicatorName+l+"label");
      }
              
//---- RETURN
return(0);

} // END DEINIT()

//+---------------------------------------------------------------------------+
//| START()                                                                   |
//+---------------------------------------------------------------------------+
int start() {

     string separator;
     int    window=WindowFind(shortName);
     int    k=0;
              
   for(int p=0; p<ArraySize(periods);p++) {
      for(int i=0; i<barsPerTimeFrame;i++,k++) {
         ExtMapBuffer1[k] = iRSI(NULL,periods[p],RSIperiod,PriceField,i);
      } // END for loop
      ExtMapBuffer1[k] =EMPTY_VALUE;
   
      k += 1;
      
      separator = indicatorName+p;
      if(ObjectFind(separator)==-1)
         ObjectCreate(separator,OBJ_TREND,window,0,0);
         ObjectSet(separator,OBJPROP_TIME1,barTime(k-Shift*(barsPerTimeFrame+1)-1));
         ObjectSet(separator,OBJPROP_TIME2,barTime(k-Shift*(barsPerTimeFrame+1)-1));
         ObjectSet(separator,OBJPROP_PRICE1,  0);
         ObjectSet(separator,OBJPROP_PRICE2,100);
         ObjectSet(separator,OBJPROP_COLOR ,separatorColor);
         ObjectSet(separator,OBJPROP_WIDTH ,2);
      separator = indicatorName+p+"label";
      if(ObjectFind(separator)==-1)
         ObjectCreate(separator,OBJ_TEXT,window,0,0);
         ObjectSet(separator,OBJPROP_TIME1,barTime(k-Shift*(barsPerTimeFrame+1)-5));
         ObjectSet(separator,OBJPROP_PRICE1,100);            
         ObjectSetText(separator,labels[p],9,"Arial",txtColor);
   } // END for loop
      
   SetIndexDrawBegin(0,Bars-k);
   SetIndexDrawBegin(1,Bars-k);  
   
//---- RETURN
return(0);

} // END START()

//+------------------------------------------------------------------+
//+ Custom functions and procedures                                  +
//+------------------------------------------------------------------+

int barTime(int a) {
   if(a<0)
         return(Time[0]+Period()*60*MathAbs(a));
   else  return(Time[a]);   
}

//+------------------------------------------------------------------+
//+                                                                  +
//+------------------------------------------------------------------+

int stringToTimeFrame(string TimeFrame) {
   int TimeFrameInt=0;
      if (TimeFrame=="M1")  TimeFrameInt=PERIOD_M1;
      if (TimeFrame=="M5")  TimeFrameInt=PERIOD_M5;
      if (TimeFrame=="M15") TimeFrameInt=PERIOD_M15;
      if (TimeFrame=="M30") TimeFrameInt=PERIOD_M30;
      if (TimeFrame=="H1")  TimeFrameInt=PERIOD_H1;
      if (TimeFrame=="H4")  TimeFrameInt=PERIOD_H4;
      if (TimeFrame=="D1")  TimeFrameInt=PERIOD_D1;
      if (TimeFrame=="W1")  TimeFrameInt=PERIOD_W1;
      if (TimeFrame=="MN")  TimeFrameInt=PERIOD_MN1;
   return(TimeFrameInt);
}

//+------------------------------------------------------------------+
//+                                                                  +
//+------------------------------------------------------------------+

string StringUpperCase(string str) {
   string   s = str;
   int      lenght = StringLen(str) - 1;
   int      char1[]; // B600 ready // Old = char;
   
   while(lenght >= 0)
      {
         char1[lenght] = StringGetChar(s, lenght);
         
         if((char1[lenght] > 96 && char1[lenght] < 123) || (char1[lenght] > 223 && char1[lenght] < 256))
                  s = StringSetChar(s, lenght, char1[lenght] - 32);
          else 
              if(char1[lenght] > -33 && char1[lenght] < 0)
                  s = StringSetChar(s, lenght, char1[lenght] + 224);
                                 
         lenght--;
   }
   
   return(s);
}
//+---------------------------------------------------------------------------+
//|                               THE END                                     |
//+---------------------------------------------------------------------------+

/* START FOOTNOTES -----------------------------------------------------------+
1. Footnotes can be removed safely if there is no want/need for them.
2. Usually used for more in-depth developers notes for members working
   on a team.
+- END FOOTNOTES -------------------------------------------------------------*/